//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "BandWidthControl.h"
#include "Emule.h"
#include "opcodes.h"
#include "preferences.h"

CBandWidthControl::CBandWidthControl()
:	m_statisticHistory(1024) // size  ~= 1024*(4*8+3*4) = 1024*44 bytes = 44 KBytes
{
	m_statistic.eMuleOutOctets = 0;
	m_statistic.eMuleInOctets = 0;
	m_statistic.eMuleOutOverallOctets = 0;
	m_statistic.eMuleInOverallOctets = 0;
	m_statistic.networkOutOctets = 0;
	m_statistic.networkInOctets = 0;
	m_statistic.timeStamp = ::GetTickCount(); 
    m_maxDownloadLimit = 0.0f;
    m_maxUploadLimit = 0.0f;
    m_errorTraced = false;

    // [TPT] - Xman
    // Keep last result to detect an overflow
    m_networkOutOctets = 0;
    m_networkInOctets = 0;
    // [TPT] - Xman

	// Cache index value
	m_currentAdapterIndex = 0;

	// Dynamic load library iphlpapi.dll => user of win95
	m_hIphlpapi = ::LoadLibrary("iphlpapi.dll");
	if(m_hIphlpapi != NULL){
		m_fGetIfTable = (GETIFTABLE)GetProcAddress(m_hIphlpapi, "GetIfTable");
        m_fGetIpAddrTable = (GETIPADDRTABLE)GetProcAddress(m_hIphlpapi, "GetIpAddrTable");
		m_fGetIfEntry = (GETIFENTRY)GetProcAddress(m_hIphlpapi, "GetIfEntry");
		m_fGetNumberOfInterfaces = (GETNUMBEROFINTERFACES)GetProcAddress(m_hIphlpapi, "GetNumberOfInterfaces");

		if (thePrefs.GetVerbose()) AddPhoenixLogLine(false, _T("NAFC: Succeed to load library iphlpapi.dll")); // [TPT] - Debug log
	}
	else {
		m_fGetIfTable = NULL;
		m_fGetIfEntry = NULL;
		m_fGetNumberOfInterfaces = NULL;

        if (thePrefs.GetVerbose()) AddPhoenixLogLine(false, _T("NAFC: Fail to load library iphlpapi.dll")); // [TPT] - Debug log
	}
}

CBandWidthControl::~CBandWidthControl(){
	// Unload library
	if(m_hIphlpapi != NULL){
		::FreeLibrary(m_hIphlpapi);
	}
}

DWORD CBandWidthControl::getAdapterIndex(){
	// Check if the library was succefully loaded
	if(m_fGetNumberOfInterfaces != NULL && m_fGetIfTable != NULL && m_fGetIpAddrTable != NULL){
		DWORD dwNumIf = 0;
		if(m_fGetNumberOfInterfaces(&dwNumIf) == NO_ERROR && dwNumIf > 0 ){
			BYTE buffer[10*sizeof(MIB_IFROW)];
			ULONG size = sizeof(buffer);
			MIB_IFTABLE& mibIfTable = reinterpret_cast<MIB_IFTABLE&>(buffer[0]);
			if(m_fGetIfTable(&mibIfTable, &size, true) == NO_ERROR){
                // Trace list of Adapters
                if(m_errorTraced == false){
				    for(DWORD dwNumEntries = 0; dwNumEntries < mibIfTable.dwNumEntries; dwNumEntries++){
                        const MIB_IFROW& mibIfRow = mibIfTable.table[dwNumEntries];
                        if (thePrefs.GetVerbose()) AddPhoenixLogLine(false, _T("NAFC: Adapter %u is '%s'"), mibIfRow.dwIndex, mibIfRow.bDescr); // [TPT] - Debug log
				    }
                }

                // Retrieve the default used IP (=> in case of multiple adapters)
                char hostName[256];
                if(gethostname(hostName, sizeof(hostName)) == 0){
                    hostent* lphost = gethostbyname(hostName);
                    if(lphost != NULL){
                        DWORD dwAddr = ((LPIN_ADDR)lphost->h_addr)->s_addr;
                        // Pick the interface matching the IP
		                BYTE buffer[10*sizeof(MIB_IPADDRROW)];
		                ULONG size = sizeof(buffer);
		                MIB_IPADDRTABLE& mibIPAddrTable = reinterpret_cast<MIB_IPADDRTABLE&>(buffer[0]);
                        if(m_fGetIpAddrTable(&mibIPAddrTable, &size, FALSE) == 0){
                            for(DWORD i = 0; i < mibIPAddrTable.dwNumEntries; i++){
                                if(mibIPAddrTable.table[i].dwAddr == dwAddr){
                                    const MIB_IPADDRROW& row = mibIPAddrTable.table[i];

                                    m_errorTraced = false;
                                    if (thePrefs.GetVerbose()) AddPhoenixLogLine(false, _T("NAFC: Select adapter with index %u"), mibIPAddrTable.table[i].dwIndex);
                                    return mibIPAddrTable.table[i].dwIndex;
                                }
                            }
                        }
                        else {
                            if(m_errorTraced == false){
                                m_errorTraced = true;
                                if (thePrefs.GetVerbose()) AddPhoenixLogLine(false, _T("NAFC: Failed to get IP tables error=0x%x"), ::GetLastError()); // [TPT] - Debug log
                                return 0;
                            }
                        }
                    }
                }
			}
            if(m_errorTraced == false){
                m_errorTraced = true;
                if (thePrefs.GetVerbose()) AddPhoenixLogLine(false, _T("NAFC: Failed to get tables of interface error=0x%x"), ::GetLastError()); // [TPT] - Debug log
                return 0;
            }
		}
        if(m_errorTraced == false){
            m_errorTraced = true;
            if (thePrefs.GetVerbose()) AddPhoenixLogLine(false, _T("NAFC: Failed to get the number of interface error=0x%x"), ::GetLastError()); // [TPT] - Debug log
            return 0;
        }
	}
	return 0;
}

void CBandWidthControl::Process()
{
    static uint16 s_counter; 
    if(++s_counter >= (1000/TIMER_PERIOD)){
        s_counter = 0;

	    // Update the datarate directly from the network Adapter
	    if(m_currentAdapterIndex == 0){
		    // Try to get the Adapter Index to access to the right interface
		    m_currentAdapterIndex = getAdapterIndex();

		    // Disable NAFC
		    if(m_currentAdapterIndex == 0){
			    thePrefs.SetNAFCEnable(false);
		    }
	    }
	    if(m_currentAdapterIndex != 0 && m_fGetIfEntry != NULL){
		    static int s_Log; // Static initiate with zero
		    MIB_IFROW ifRow;        
		    ifRow.dwIndex = m_currentAdapterIndex;
		    if(m_fGetIfEntry(&ifRow) == NO_ERROR){
		        s_Log = 0;

	       	   // [TPT] - Xman
               // Add the delta, since the last measure (convert 32 to 64 bits)
               m_statistic.networkInOctets += (DWORD)(ifRow.dwInOctets - m_networkOutOctets);
               m_statistic.networkOutOctets += (DWORD)(ifRow.dwOutOctets - m_networkInOctets);

               // Keep last measure
               m_networkOutOctets = ifRow.dwInOctets;
               m_networkInOctets = ifRow.dwOutOctets;
               // [TPT] - Xman
               
		    }
		    else {
		        if(s_Log == 0){
			        s_Log = 1;
			        if (thePrefs.GetVerbose()) AddPhoenixLogLine(false, _T("NAFC: Failed to retrieve adapter traffic, error=0x%x"), ::GetLastError()); // [TPT] - Debug log
                }

			    // Disable NAFC
			    thePrefs.SetNAFCEnable(false);
		    }
	    }

	    // Update large history list for the history graph
	    m_statisticHistory.AddHead(Statistic(m_statistic, ::GetTickCount()));
        const uint32 averageMinTime = (uint32)(60000 * thePrefs.GetStatsAverageMinutes());
	    while(true){
		    // Avoid to use the list's size. The timestamp is more accurate than the periode of Process()
		    const uint32 deltaTime = m_statisticHistory.GetHead().timeStamp - m_statisticHistory.GetTail().timeStamp;
		    if(deltaTime <= averageMinTime){
                break; // exit loop
            }
		    m_statisticHistory.RemoveTail(); // Trunk size
	    }

        // Calculate the dynamic dowload limit
        m_maxDownloadLimit = thePrefs.GetMaxDownload();
        m_maxUploadLimit = thePrefs.GetMaxUpload();
		
		// [TPT] - Xman Bugfix
		if(thePrefs.GetNAFCEnable() == true && thePrefs.GetNAFCFullControl() == true )
		{	   
  		  if(m_statisticHistory.GetSize() > 1) // [TPT] - Xman Bugfix
		  {
		    const Statistic& newestSample = m_statisticHistory.GetHead();
            const Statistic& oldestSample = m_statisticHistory.GetAt(m_statisticHistory.FindIndex(1));
		    const uint32 deltaTime = (newestSample.timeStamp - oldestSample.timeStamp); // in [ms]
			if (deltaTime == 0) return;
            const uint32 eMuleOutOverall = (1000 * (uint32)(newestSample.eMuleOutOverallOctets - oldestSample.eMuleOutOverallOctets) / deltaTime); // in [Bytes/s]
            const uint32 networkOut = (1000 * (uint32)(newestSample.networkOutOctets - oldestSample.networkOutOctets) / deltaTime); // in [Bytes/s]

            m_maxUploadLimit = thePrefs.GetNAFCNetworkOut();

            // Dynamic limit with ratio
            float maxDownloadLimit;
            if(eMuleOutOverall < 4*1024)
			{
                // Ratio 3x
                maxDownloadLimit = (float)(3*eMuleOutOverall) / 1024.0f; // [KB/s]
                if(maxDownloadLimit < m_maxDownloadLimit)
                    m_maxDownloadLimit = maxDownloadLimit;
                }
            else if(eMuleOutOverall < 10*1024)
			{
                // Ratio 4x
                maxDownloadLimit = (float)(4*eMuleOutOverall) / 1024.0f; // [KB/s]
                if(maxDownloadLimit < m_maxDownloadLimit)
                    m_maxDownloadLimit = maxDownloadLimit;
                }

            if(m_maxDownloadLimit < 1.0f)
                m_maxDownloadLimit = 1.0f;

            if(thePrefs.GetBlockMaellaSpecificDebugMsg() == false && 
                thePrefs.GetNAFCEnable() == true && 
                thePrefs.GetNAFCFullControl() == true)
			{
                 if (thePrefs.GetVerbose()) AddPhoenixLogLine(false, _T("NAFC: new download/upload limits %0.1f/%0.1f [KB/s] for an upload rate eMule(Data+Control)/Adapter %u/%u [bytes/s]"),  // [TPT] - Debug log
                                                  GetMaxDownload(), 
                                                  GetMaxUpload(),
                                                  eMuleOutOverall,
                                                  networkOut);
		    }
		  }
		}
		// [TPT] - Xman Bugfix
	}    
    else if(thePrefs.GetNAFCEnable() == true && thePrefs.GetNAFCFullControl() == true ){
        // Retrieve the network flow => necessary for the NAFC with 'Auto U/D limit'
	    if(m_currentAdapterIndex != 0 && m_fGetIfEntry != NULL){
		    MIB_IFROW ifRow;        
		    ifRow.dwIndex = m_currentAdapterIndex;
		    if(m_fGetIfEntry(&ifRow) == NO_ERROR){

			  // [TPT] - Xman
              // Add the delta, since the last measure (convert 32 to 64 bits)
              m_statistic.networkInOctets += (DWORD)(ifRow.dwInOctets - m_networkOutOctets);
              m_statistic.networkOutOctets += (DWORD)(ifRow.dwOutOctets - m_networkInOctets);

              // Keep last measure
              m_networkOutOctets = ifRow.dwInOctets;
              m_networkInOctets = ifRow.dwOutOctets;
              // [TPT] - Xman
		    }
	    }
    }
}

void CBandWidthControl::GetDatarates(uint8 samples, 
									 uint32& eMuleIn, uint32& eMuleInOverall, 
									 uint32& eMuleOut, uint32& eMuleOutOverall,
									 uint32& networkIn, uint32& networkOut) const 
{
	// Check if the list is already long enough
	if(m_statisticHistory.GetSize() > 2 && samples >= 1){
		// Retieve the location of the n previous sample
		POSITION pos = m_statisticHistory.FindIndex(samples);
		if(pos == NULL){
			pos = m_statisticHistory.GetTailPosition();
		}
			
		const Statistic& newestSample = m_statisticHistory.GetHead();
		const Statistic& oldestSample = m_statisticHistory.GetAt(pos);
		const uint32 deltaTime = (newestSample.timeStamp - oldestSample.timeStamp); // in [ms]
		
		if(deltaTime > 0){
			eMuleIn = (uint32)(1000 * (newestSample.eMuleInOctets - oldestSample.eMuleInOctets) / deltaTime); // in [Bytes/s]
			eMuleInOverall = (uint32)(1000 * (newestSample.eMuleInOverallOctets - oldestSample.eMuleInOverallOctets) / deltaTime); // in [Bytes/s]
			eMuleOut = (uint32)(1000 * (newestSample.eMuleOutOctets - oldestSample.eMuleOutOctets) / deltaTime); // in [Bytes/s]
			eMuleOutOverall = (uint32)(1000 * (newestSample.eMuleOutOverallOctets - oldestSample.eMuleOutOverallOctets) / deltaTime); // in [Bytes/s]
			networkIn = (uint32)(1000 * (newestSample.networkInOctets - oldestSample.networkInOctets) / deltaTime); // in [Bytes/s]
			networkOut = (uint32)(1000 * (newestSample.networkOutOctets - oldestSample.networkOutOctets) / deltaTime); // in [Bytes/s]
			
			return; // over
		}
	}

	eMuleIn = 0;
	eMuleInOverall = 0;
	eMuleOut = 0;
	eMuleOutOverall = 0;
	networkIn = 0;
	networkOut = 0;
}

void CBandWidthControl::GetFullHistoryDatarates(uint32& eMuleInHistory, uint32& eMuleOutHistory, 
												uint32& eMuleInSession, uint32& eMuleOutSession) const {

	eMuleInHistory = 0;
	eMuleOutHistory = 0;
	eMuleInSession = 0;
	eMuleOutSession = 0;

	// Check if the list is already long enough
	if(m_statisticHistory.GetSize() >= 2){

		const Statistic& newestSample = m_statisticHistory.GetHead();
		const Statistic& oldestSample = m_statisticHistory.GetTail();

		// Average value since the last n minutes
		uint32 deltaTime = (newestSample.timeStamp - oldestSample.timeStamp); // in [ms]
		if(deltaTime > 0){
			eMuleInHistory = (uint32)(1000 * (newestSample.eMuleInOctets - oldestSample.eMuleInOctets) / deltaTime); // in [Bytes/s]
			eMuleOutHistory = (uint32)(1000 * (newestSample.eMuleOutOctets - oldestSample.eMuleOutOctets) / deltaTime); // in [Bytes/s]
		}

		// Average value since the start of the client
		deltaTime = (::GetTickCount() - GetStartTick()) / 1000; // in [s]
		if(deltaTime > 0){
			eMuleInSession = (uint32)(newestSample.eMuleInOctets / deltaTime); // in [Bytes/s]
			eMuleOutSession = (uint32)(newestSample.eMuleOutOctets / deltaTime); // in [Bytes/s]
		}
	}
}