#include "StdAfx.h"
#include "ininotifier.h"

CIniNotifier::CIniNotifier(void)
{
}

CIniNotifier::~CIniNotifier(void)
{
}

CIniNotifier::CIniNotifier(const CIniNotifier& itbn) {
	//m_nIniVersion=itbn.;
	m_bValidConfig=itbn.m_bValidConfig;
	m_strSkinDir=itbn.m_strSkinDir;
	m_colorNormalText=itbn.m_colorNormalText;	
	m_colorSelectedText=itbn.m_colorSelectedText;	
	m_colorBmpTransparent=itbn.m_colorBmpTransparent;	
	m_rcTextArea=itbn.m_rcTextArea;    
	m_rcCloseBtnArea=itbn.m_rcCloseBtnArea;	
	m_rcHistoryBtnArea=itbn.m_rcHistoryBtnArea;    
	m_dwTimeToStay=itbn.m_dwTimeToStay;
	m_dwTimeToShow=itbn.m_dwTimeToShow;
	m_dwTimeToHide=itbn.m_dwTimeToHide;   
	m_strFontType=itbn.m_strFontType;
	m_nFontSize=itbn.m_nFontSize;
	m_strBmpFileName=itbn.m_strBmpFileName;
}

bool CIniNotifier::Open(CString skinFile, bool readOnly) 
{
	if (skinFile.GetLength() > 255 || skinFile.IsEmpty()) {
		m_bValidConfig = false;
		return false;
	}
		
    SetFileName(skinFile);
    SetSection("CONFIG");

	m_strSkinDir = skinFile.Left(skinFile.ReverseFind('\\'));	

	LoadConfiguration();
	m_bValidConfig=true;
	return true;
}

bool CIniNotifier::Save(CString skinFile)
{
	if (GetReadOnly()) return false;

    ASSERT(skinFile);
	SetFileName(skinFile);
	SetSection("CONFIG");
	SaveConfiguration();
	return false;
}

void CIniNotifier::LoadConfiguration()
{	
	uint8 nRed, nGreen, nBlue, sRed, sGreen, sBlue;
	uint16 rcLeft, rcTop, rcRight, rcBottom;	
	uint8 bmpTrasparentRed, bmpTrasparentGreen, bmpTrasparentBlue;
	CString fontType, bmpFullPath;

	nRed   = GetInt("TextNormalRed",255);
	nGreen = GetInt("TextNormalGreen",255);
	nBlue  = GetInt("TextNormalBlue",255);

	SetNormalTextColor(nRed, nGreen, nBlue);

	sRed   = GetInt("TextSelectedRed",255);
	sGreen = GetInt("TextSelectedGreen",255);
	sBlue  = GetInt("TextSelectedBlue",255);

	SetSelectedTextColor(sRed, sGreen, sBlue);

	bmpTrasparentRed   = GetInt("bmpTrasparentRed",255);
	bmpTrasparentGreen = GetInt("bmpTrasparentGreen",0);
	bmpTrasparentBlue  = GetInt("bmpTrasparentBlue",255);

	SetBmpTransparentColor(bmpTrasparentRed, bmpTrasparentGreen, bmpTrasparentBlue);

	SetFontSize(GetInt("TextFontSize",70));
	SetTimeToStay(GetInt("TimeToStay", 4000));
	SetTimeToShow(GetInt("TimeToShow", 500));
	SetTimeToHide(GetInt("TimeToHide", 200));
	//TODO: cannot pass GetString as SetFontType parameter: it cause a crash but only in release build. why??
	fontType=GetString("FontType", "Arial");
	SetFontType(fontType);
	bmpFullPath.Format("%s\\%s", m_strSkinDir, GetString("bmpFileName", ""));
	SetBmpFileName(bmpFullPath);

	// get text rectangle coordinates
	
	rcLeft = GetInt("rcTextLeft",1);
	rcTop  = GetInt("rcTextTop",1);	
	rcRight  = GetInt("rcTextRight", 1);
	rcBottom = GetInt("rcTextBottom", 1);

	SetTextArea(rcLeft,rcTop,rcRight,rcBottom);

	// get close button rectangle coordinates
	rcLeft = GetInt("rcCloseBtnLeft",1);
	rcTop  = GetInt("rcCloseBtnTop",1);	
	rcRight  = GetInt("rcCloseBtnRight", 1);
	rcBottom = GetInt("rcCloseBtnBottom", 1);

	SetCloseBtnArea(rcLeft,rcTop,rcRight,rcBottom);
	
	// get history button rectangle coordinates
	rcLeft = GetInt("rcHistoryBtnLeft",1);
	rcTop  = GetInt("rcHistoryBtnTop",1);	
	rcRight  = GetInt("rcHistoryBtnRight", 1);
	rcBottom = GetInt("rcHistoryBtnBottom", 1);

	SetHistoryBtnArea(rcLeft,rcTop,rcRight,rcBottom);
}

void CIniNotifier::SaveConfiguration()
{	
	WriteInt("TextNormalRed",GetRValue(GetNormalTextColor()));
	WriteInt("TextNormalGreen",GetGValue(GetNormalTextColor()));
	WriteInt("TextNormalBlue",GetBValue(GetNormalTextColor()));	

	WriteInt("TextSelectedRed",GetRValue(GetSelectedTextColor()));
	WriteInt("TextSelectedGreen",GetGValue(GetSelectedTextColor()));
	WriteInt("TextSelectedBlue",GetBValue(GetSelectedTextColor()));

	WriteInt("bmpTrasparentRed",GetRValue(GetBmpTransparentColor()));
	WriteInt("bmpTrasparentGreen",GetGValue(GetBmpTransparentColor()));
	WriteInt("bmpTrasparentBlue",GetBValue(GetBmpTransparentColor()));	

	WriteInt("TextFontSize",GetFontSize());
	WriteInt("TimeToStay", GetTimeToStay());
	WriteInt("TimeToShow", GetTimeToShow());
	WriteInt("TimeToHide", GetTimeToHide());
	WriteString("FontType", GetFontType());
	WriteString("bmpFileName", GetBmpFileName());		

	WriteInt("rcTextLeft",GetTextArea().TopLeft().x);
	WriteInt("rcTextTop",GetTextArea().TopLeft().y);	
	WriteInt("rcTextRight", GetTextArea().BottomRight().x);
	WriteInt("rcTextBottom", GetTextArea().BottomRight().y);

	WriteInt("rcCloseBtnLeft",GetCloseBtnArea().TopLeft().x);
	WriteInt("rcCloseBtnTop",GetCloseBtnArea().TopLeft().y);	
	WriteInt("rcCloseBtnRight", GetCloseBtnArea().BottomRight().x);
	WriteInt("rcCloseBtnBottom", GetCloseBtnArea().BottomRight().y);

	WriteInt("rcHistoryBtnLeft",GetHistoryBtnArea().TopLeft().x);
	WriteInt("rcHistoryBtnTop",GetHistoryBtnArea().TopLeft().y);	
	WriteInt("rcHistoryBtnRight", GetHistoryBtnArea().BottomRight().x);
	WriteInt("rcHistoryBtnBottom", GetHistoryBtnArea().BottomRight().y);
}
